
# q-expansion of newform 1624.2.a.c, downloaded from the LMFDB on 17 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1624
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [-1], [-6], [4], [6], [-5], [-1], [-1], [8], [-8], [-7], [-10], [-5], [-11], [0], [2], [9], [-15], [3], [-12], [6], [11], [-1], [5], [8], [-7], [11], [12], [-18], [-8], [-4], [14], [-13], [9], [-3], [8], [0], [-24], [-12], [14], [20], [8], [19], [4], [-8], [-2], [12], [5], [-3], [-8], [8], [13], [-20], [-18], [-25], [-7], [-3], [18], [8], [-21], [-28], [25], [-16], [6], [6], [20], [-20], [16], [36], [-14], [-23], [-10], [-26], [10], [16], [28], [1], [27], [-18], [-20], [5], [25], [34], [10], [-14], [-17], [-18], [19], [-5], [17], [-11], [12], [-5], [16], [-38], [-14], [0], [-10], [-45], [5], [-32], [-38], [39], [46], [-42], [-6], [-30], [-41], [7], [11], [-30], [44], [5], [-6], [32], [2], [10], [8], [-2], [-1], [9], [-28], [-26], [-51], [26], [-28], [-37], [1], [10], [28], [-10], [12], [-18], [54], [-45], [-44], [-18], [36], [48], [-30], [-6], [28], [-47], [9], [6], [-24], [-32], [9], [-41], [-43], [-27], [-3], [26], [-40], [-35], [-34], [-2], [-42], [-48], [43], [-32], [12], [-42], [24], [-59], [-3], [37], [-45], [51], [-28], [-24], [18], [24], [44], [48], [-28], [56], [-44], [12], [9], [21], [34], [48], [16], [-26], [31], [-14], [18], [62], [-62], [-4], [34], [26], [56], [40], [-31], [6], [44], [-50], [-36], [-10], [-49], [-7], [26], [7], [43], [-33], [-17], [2], [0], [22], [36], [-21], [12], [-47], [-31], [-33], [-2], [-56], [52], [-62], [42], [-4], [-40], [-57], [18], [48], [-49], [-50], [74], [-18], [32], [-18], [32], [38], [2], [4], [49], [-6], [-22], [62], [16], [54], [18], [-38], [66], [-53], [54], [-50], [18], [-68], [-71], [-19], [33], [44], [13], [16], [-30], [-64], [60], [68], [-61], [-75], [-61], [-6], [-30], [28], [2], [42], [35], [28], [-80], [61], [-21], [-8], [-21], [-46], [18], [-30], [-16], [10], [-10], [1], [-8], [-28], [18], [32], [38], [10], [-42], [-61], [60], [64], [-46], [-5], [70], [56], [-52], [15], [3], [-52], [24], [-48], [55], [16], [-30], [-24], [-22], [-2], [-32], [-21], [-52], [43], [-84], [0], [29], [26], [-56], [68], [75], [36], [-34], [-33], [14], [-3], [66], [0], [71], [68], [-23], [66], [55], [-48], [-20], [65], [-66], [-23], [28], [-26], [70], [59], [-56], [18], [-22], [-74], [-4], [-75], [-42], [42], [-50], [-62], [46], [-62], [54], [9], [8], [28], [-16], [-73], [-22], [-38], [73], [-48], [-46], [-55], [-28], [54], [50], [-20], [13], [66], [-74], [-3], [93], [70], [35], [84], [-42], [-6], [-39], [2], [-74], [86], [-18], [-24], [-8], [-20], [-92], [40], [-54], [48], [29], [6], [-73], [-64], [-54], [-23], [2], [-14], [-84], [-46], [8], [42], [78], [-18], [-48], [-79], [-22], [10], [54], [93], [28], [-34], [35], [-70], [-34], [9], [48], [-49], [38], [42], [-9], [-28], [-50], [20], [70], [60], [-39]]
